# Definitions of names of inflected forms for Finnish.  This file also contains
# iterators for enumerating all 5-tuples for possible inflectional forms for
# any given part-of-speech.
#
# Copyright (c) 2018 Tatu Ylonen.  See LICENSE.

# Names of comparative forms for adjectives (the empty string means positive,
# or normal form).
COMP_FORMS = ("", "comp", "sup")

# Names of Finnmish cases.  We combine case and number into the same name.
CASE_FORMS = (
    "nom-sg",
    "acc-sg",  # Used for certain pronouns only
    "gen-sg",
    "ptv-sg",
    "ine-sg",
    "ela-sg",
    "ill-sg",
    "ade-sg",
    "abl-sg",
    "all-sg",
    "ess-sg",
    "tra-sg",
    "ins-sg",
    "abe-sg",
    "cmt-sg",
    "nom-pl",
    "acc-pl",  # Used for certain pronouns only
    "gen-pl",
    "ptv-pl",
    "ine-pl",
    "ela-pl",
    "ill-pl",
    "ade-pl",
    "abl-pl",
    "all-pl",
    "ess-pl",
    "tra-pl",
    "ins-pl",
    "abe-pl",
    "cmt-pl")

# Names of possessive suffixes.  The empty string means no possessive suffix.
# 3x means either plural or singular third person (they have the same form).
POSSESSIVE_FORMS = ("", "1s", "2s", "3x", "1p", "2p")

# Names of verb forms.
#
# XXX should perhaps combine some of the negative forms as they are usually
# (always?) identical.  Check whether they really are always.
VERB_FORMS = (
    "pres-1sg",
    "pres-2sg",
    "pres-3sg",
    "pres-1pl",
    "pres-2pl",
    "pres-3pl",
    "pres-1sg-neg",
    "pres-2sg-neg",
    "pres-3sg-neg",
    "pres-1pl-neg",
    "pres-2pl-neg",
    "pres-3pl-neg",
    "pres-pass",
    "pres-pass-neg",
    "past-1sg",
    "past-2sg",
    "past-3sg",
    "past-1pl",
    "past-2pl",
    "past-3pl",
    "past-1sg-neg",
    "past-2sg-neg",
    "past-3sg-neg",
    "past-1pl-neg",
    "past-2pl-neg",
    "past-3pl-neg",
    "past-pass",
    "past-pass-neg",
    "cond-1sg",
    "cond-2sg",
    "cond-3sg",
    "cond-1pl",
    "cond-2pl",
    "cond-3pl",
    "cond-1sg-neg",
    "cond-2sg-neg",
    "cond-3sg-neg",
    "cond-1pl-neg",
    "cond-2pl-neg",
    "cond-3pl-neg",
    "cond-pass",
    "cond-pass-neg",
    "impr-2sg",
    "impr-3sg",
    "impr-1pl",
    "impr-2pl",
    "impr-3pl",
    "impr-2sg-neg",
    "impr-3sg-neg",
    "impr-1pl-neg",
    "impr-2pl-neg",
    "impr-3pl-neg",
    "impr-pass",
    "impr-pass-neg",
    "potn-1sg",
    "potn-2sg",
    "potn-3sg",
    "potn-1pl",
    "potn-2pl",
    "potn-3pl",
    "potn-1sg-neg",
    "potn-2sg-neg",
    "potn-3sg-neg",
    "potn-1pl-neg",
    "potn-2pl-neg",
    "potn-3pl-neg",
    "potn-pass",
    "potn-pass-neg",
    "pres-part",
    "pres-pass-part",
    "past-part",
    "past-pass-part",
    "agnt-part",
    "nega-part",
    "inf1",
    "inf1-long",
    "inf2-ine",
    "inf2-pass-ine",
    "inf2-ins",
    "inf3-ine",
    "inf3-ela",
    "inf3-ill",
    "inf3-ade",
    "inf3-abe",
    "inf3-ins",
    "inf3-pass-ins",
    "inf4-nom",
    "inf4-par",
    "inf5",
)

# Names of all clitics and allowed clitic combinations.  The empty string
# means no clitic.
CLITIC_FORMS = ("",
                "kO",
                "kin",
                "kAAn",
                "pA",
                "s",
                "kA",
                "hAn",
                "kOhAn",
                "pAhAn",
                "pAs",
                "kOs",
                "kinkO",
                "kAAnkO",
                "kinkOhAn")

######################################################################
# The rest of file is about enumerating 5-tuples representing
# inflected forms.
######################################################################

# Cache of all forms for combinations of part-of-speech and other parameters
# restricting which forms are posible.
all_forms_cache = {}


def all_forms_list(*args, **kwargs):
    """Returns a list/tuple of all possible word form descriptors for the given
    part-of-speech and other restrictions.  This caches the result."""
    key = (args, tuple(sorted(kwargs.items())))
    if key in all_forms_cache:
        return all_forms_cache[key]
    forms = tuple(x for x in all_forms_iter(*args, **kwargs))
    all_forms_cache[key] = forms
    return forms


def all_forms_iter(pos, comparable=True, transitive=True, no_clitics=False):
    """Returns an iterator that iterates over all possible word form
    descriptors for the given part-of-speech and other parameters."""
    assert isinstance(pos, str)

    clitics = [""] if no_clitics else CLITIC_FORMS

    def fn():
        if pos in ("noun", "name", "num", "letter"):
            for case in CASE_FORMS:
                for poss in POSSESSIVE_FORMS:
                    if case in ("acc-sg", "acc-pl"):
                        continue
                    if case == "cmt-pl" and not poss:
                        continue
                    for clitic in clitics:
                        if clitic in ('s', 'kA'):
                            continue
                        yield ("", "", case, poss, clitic)
        elif pos == "pron":
            # XXX other types of pronouns?  This is for perspron.
            for case in CASE_FORMS:
                if case in ("ins-sg", "ins-pl", "cmt"):
                    continue
                for clitic in clitics:
                    if clitic in ("s", "kA"):
                        continue
                    yield ("", "", case, '', clitic)
        elif pos == "adj":
            comps = COMP_FORMS if comparable else [""]
            for comp in comps:
                for case in CASE_FORMS:
                    if case in ("acc-sg", "acc-pl"):
                        continue
                    for poss in POSSESSIVE_FORMS:
                        for clitic in clitics:
                            if clitic in ('s',):
                                continue
                            yield ("", comp, case, poss, clitic)
        elif pos == "verb":
            for vform in VERB_FORMS:
                if not transitive and vform == "agnt-part":
                    continue
                comps = [""]
                cases = [""]
                posses = [""]
                if vform in ("pres-part", "pres-pass-part",
                             "past-part", "past-pass-part",
                             "agnt-part", "nega-part"):
                    cases = CASE_FORMS
                if vform in ("inf1-long", "inf2-ine",
                             "inf3-ine", "inf3-ela",
                             "inf3-ill", "inf3-ade",
                             "inf3-abe", "inf4-par",
                             "inf5", "agnt-part", "pres-pass-part",
                             "pres-part"):
                    posses = POSSESSIVE_FORMS
                if vform in ("pres-part", "past-part",
                             "pres-pass-part", "past-pass-part"):
                    comps = COMP_FORMS if comparable else [""]
                for comp in comps:
                    for case in cases:
                        if case in ("acc-sg", "acc-pl"):
                            continue
                        for poss in posses:
                            if not poss and vform in ("inf1-long", "inf5"):
                                continue
                            if (poss and case != "nom-sg" and
                                vform in ("pres-part", "past-pass-part")):
                                continue
                            for clitic in clitics:
                                yield (vform, comp, case, poss, clitic)
        elif pos in ("conj", "intj", "suffix", "clitic", "punct"):
            yield ("", "", "", "", "")
        elif pos in ("postp", "prep", "adv"):
            for clitic in clitics:
                yield ("", "", "", "", clitic)
                if pos == "adv":
                    yield ("", "comp", "", "", clitic)
                    yield ("", "sup", "", "", clitic)
        else:
            print("all_forms_iter: unimplemented pos:", pos)
            yield ("", "", "", "", "")
    return fn()



#!/usr/bin/env python3

import setuptools

with open("README.md", "r") as f:
    long_description = f.read()

from distutils.core import setup
setup(name="wiktfinnish",
      version="0.1.1",
      description="Finnish morphology (including verb forms, comparatives, cases, possessives, clitics)",
      long_description=long_description,
      long_description_content_type="text/markdown",
      author="Tatu Ylonen",
      author_email="ylo@clausal.com",
      url="https://clausal.com",
      license="MIT",
      download_url="https://github.com/tatuylonen/wiktfinnish",
      packages=["wiktfinnish"],
      # install_requires=[],
      classifiers=[
          "Development Status :: 3 - Alpha",
          "Intended Audience :: Developers",
          "Intended Audience :: Science/Research",
          "License :: OSI Approved :: MIT License",
          "Natural Language :: Finnish",
          "Operating System :: OS Independent",
          "Operating System :: POSIX :: Linux",
          "Programming Language :: Python",
          "Programming Language :: Python :: 3.6",
          "Programming Language :: Python :: 3.7",
          "Programming Language :: Python :: 3 :: Only",
          "Topic :: Text Processing",
          "Topic :: Text Processing :: Linguistic",
          ])



# -*- fundamental -*-
#
# The long dictionary constants in this file cause emacs Python mode to
# hang for seconds or minutes every now and then, and thus editing in
# fundamental mode is much more convenient...

# Finnish noun declension definitions.  Possessive suffixes are also defined
# in this file.  These are also used for inflecting certain verb forms.
#
# This file defines comparatives, but in general it is better to use known
# comparative forms (e.g., as extracted from Wiktionary or learned otherwise),
# as some derivations are irregular.  The comparative rules here have not
# been extensively tested.

# Variants with -poss are used if a possessive suffix or clitic follows.
# Variants with -clitic are used if a clitic but no possessive suffix follows.
# Otherwise defaults to a the rule given by the form name.  The value may
# be either a string or a list of strings, most preferred one first, or None
# if that form does not exist for the declension.
#
# Special chracters in rules include:
#   digit - use value of that argument from the declension/conjugation
#   9 - takes partitive singular vowel from par_sg_a or last deleted char
#   O - converted to o or ö based on vowel harmony
#   U - converted to u or y based on vowel harmony
#   A - converted to a or ä based on vowel harmony (usually digit used instead)
#   D - converted to D or previous character after any of: lmnr
#   @ - used for illative singular (usually vowel lengthening + special rules)
#   - - deletes previos char but it is still used for vowel harmony
#   / - deletes the char before last, but it is still used for vowel harmony
#
# Copyright (c) 2018 Tatu Ylonen.  See COPYING.

decl_name_map = {
    "fi-decl-kauneus": "fi-decl-kalleus",
    "fi-decl-nuori": "fi-decl-pieni",
}

noun_decls = {
    "fi-decl-valo": {
    	"nargs": 5,
        "nom-sg": "124",
        "gen-sg": "134n",
        "gen-sg-poss": "124",
        "ptv-sg": "1245",
        "ine-sg": "134ss5",
        "ela-sg": "134st5",
        "ill-sg": "124@n",
        "ill-sg-poss": "124@",
        "ade-sg": "134ll5",
        "abl-sg": "134lt5",
        "all-sg": "134lle",
        "ess-sg": "124n5",
        "tra-sg": "134ksi",
        "tra-sg-poss": "134kse",
        "ins-sg": None,
        "abe-sg": "134tt5",
        "cmt-sg": None,
        "nom-pl": "134t",
        "nom-pl-poss": "124",
        "gen-pl": "124jen",
        "gen-pl-poss": "124je",
        "ptv-pl": "124j5",
        "ine-pl": "134iss5",
        "ela-pl": "134ist5",
        "ill-pl": "124ihin",
        "ill-pl-poss": "124ihi",
        "ade-pl": "134ill5",
        "abl-pl": "134ilt5",
        "all-pl": "134ille",
        "ess-pl": "124in5",
        "tra-pl": "134iksi",
        "tra-pl-poss": "134ikse",
        "ins-pl": "134in",
        "ins-pl-poss": None,
        "abe-pl": "134itt5",
        "cmt-pl": "124ine",
        "comp": "134mpi",
        "sup": "134in",
    },
    "fi-decl-palvelu": {
    	"nargs": 2,
        "nom-sg": "1",
        "gen-sg": "1n",
        "gen-sg-poss": "1",
        "ptv-sg": "12",
        "ine-sg": "1ss2",
        "ela-sg": "1st2",
        "ill-sg": "1@n",
        "ill-sg-poss": "1@",
        "ade-sg": "1ll2",
        "abl-sg": "1lt2",
        "all-sg": "1lle",
        "ess-sg": "1n2",
        "tra-sg": "1ksi",
        "tra-sg-poss": "1kse",
        "ins-sg": None,
        "abe-sg": "1tt2",
        "cmt-sg": None,
        "nom-pl": "1t",
        "nom-pl-poss": "1",
        "gen-pl": ["1jen", "1iden", "1itten"],
        "gen-pl-poss": ["1je", "1ide", "1itte"],
        "ptv-pl": ["1j2", "1it2"],
        "ine-pl": "1iss2",
        "ela-pl": "1ist2",
        "ill-pl": "1ihin",
        "ill-pl-poss": "1ihi",
        "ade-pl": "1ill2",
        "abl-pl": "1ilt2",
        "all-pl": "1ille",
        "ess-pl": "1in2",
        "tra-pl": "1iksi",
        "tra-pl-poss": "1ikse",
        "ins-pl": "1in",
        "ins-pl-poss": None,
        "abe-pl": "1itt2",
        "cmt-pl": "1ine",
        "comp": "1mpi",
        "sup": "1in",
    },
    "fi-decl-valtio": {
    	"nargs": 2,
        "nom-sg": "1",
        "gen-sg": "1n",
        "gen-sg-poss": "1",
        "ptv-sg": "1t2",
        "ine-sg": "1ss2",
        "ela-sg": "1st2",
        "ill-sg": "1@n",
        "ill-sg-poss": "1@",
        "ade-sg": "1ll2",
        "abl-sg": "1lt2",
        "all-sg": "1lle",
        "ess-sg": "1n2",
        "tra-sg": "1ksi",
        "tra-sg-poss": "1kse",
        "ins-sg": None,
        "abe-sg": "1tt2",
        "cmt-sg": None,
        "nom-pl": "1t",
        "nom-pl-poss": "1",
        "gen-pl": ["1iden", "1itten"],
        "gen-pl-poss": ["1ide", "1itte"],
        "ptv-pl": "1it2",
        "ine-pl": "1iss2",
        "ela-pl": "1ist2",
        "ill-pl": "1ihin",
        "ill-pl-poss": "1ihi",
        "ade-pl": "1ill2",
        "abl-pl": "1ilt2",
        "all-pl": "1ille",
        "ess-pl": "1in2",
        "tra-pl": "1iksi",
        "tra-pl-poss": "1ikse",
        "ins-pl": "1in",
        "ins-pl-poss": None,
        "abe-pl": "1itt2",
        "cmt-pl": "1ine",
        "comp": "1mpi",
        "sup": "1in",
    },
    "fi-decl-laatikko": {
    	"nargs": 5,
        "nom-sg": "124",
        "gen-sg": "134n",
        "gen-sg-poss": "124",
        "ptv-sg": "1245",
        "ine-sg": "134ss5",
        "ela-sg": "134st5",
        "ill-sg": "124@n",
        "ill-sg-poss": "124@",
        "ade-sg": "134ll5",
        "abl-sg": "134lt5",
        "all-sg": "134lle",
        "ess-sg": "124n5",
        "tra-sg": "134ksi",
        "tra-sg-poss": "134kse",
        "ins-sg": None,
        "abe-sg": "134tt5",
        "cmt-sg": None,
        "nom-pl": "134t",
        "nom-pl-poss": "124",
        "gen-pl": ["124jen", "134iden", "134itten"],
        "gen-pl-poss": ["124je", "134ide", "134itte"],
        "ptv-pl": ["124j5", "134it5"],
        "ine-pl": "134iss5",
        "ela-pl": "134ist5",
        "ill-pl": ["124ihin", "134ihin"],
        "ill-pl-poss": ["124ihi", "134ihi"],
        "ade-pl": "134ill5",
        "abl-pl": "134ilt5",
        "all-pl": "134ille",
        "ess-pl": "124in5",
        "tra-pl": "134iksi",
        "tra-pl-poss": "134ikse",
        "ins-pl": "134in",
        "ins-pl-poss": None,
        "abe-pl": "134itt5",
        "cmt-pl": "124ine",
        "comp": "134mpi",
        "sup": "124in",
    },
    "fi-decl-risti": {
    	"nargs": 4,
        "nom-sg": "12i",
        "gen-sg": "13in",
        "gen-sg-poss": "12i",
        "ptv-sg": "12i4",
        "ine-sg": "13iss4",
        "ela-sg": "13ist4",
        "ill-sg": "12iin",
        "ill-sg-poss": "12ii",
        "ade-sg": "13ill4",
        "abl-sg": "13ilt4",
        "all-sg": "13ille",
        "ess-sg": "12in4",
        "tra-sg": "13iksi",
        "tra-sg-poss": "13ikse",
        "ins-sg": None,
        "abe-sg": "13itt4",
        "cmt-sg": None,
        "nom-pl": "13it",
        "nom-pl-poss": "12i",
        "gen-pl": "12ien",
        "gen-pl-poss": "12ie",
        "ptv-pl": "12ej4",
        "ine-pl": "13eiss4",
        "ela-pl": "13eist4",
        "ill-pl": "12eihin",
        "ill-pl-poss": "12eihi",
        "ade-pl": "13eill4",
        "abl-pl": "13eilt4",
        "all-pl": "13eille",
        "ess-pl": "12ein4",
        "tra-pl": "13eiksi",
        "tra-pl-poss": "13eikse",
        "ins-pl": "13ein",
        "ins-pl-poss": None,
        "abe-pl": "13eitt4",
        "cmt-pl": "12eine",
        "comp": "13impi",
        "sup": "13ein",
    },
    "fi-decl-paperi": {
    	"nargs": 2,
        "nom-sg": "1i",
        "gen-sg": "1in",
        "gen-sg-poss": "1i",
        "ptv-sg": "1i2",
        "ine-sg": "1iss2",
        "ela-sg": "1ist2",
        "ill-sg": "1iin",
        "ill-sg-poss": "1ii",
        "ade-sg": "1ill2",
        "abl-sg": "1ilt2",
        "all-sg": "1ille",
        "ess-sg": "1in2",
        "tra-sg": "1iksi",
        "tra-sg-poss": "1ikse",
        "ins-sg": None,
        "abe-sg": "1itt2",
        "cmt-sg": None,
        "nom-pl": "1it",
        "nom-pl-poss": "1i",
        "gen-pl": ["1ien", "1eiden", "1eitten"],
        "gen-pl-poss": ["1ie", "1eide", "1eitte"],
        "ptv-pl": ["1ej2", "1eit2"],
        "ine-pl": "1eiss2",
        "ela-pl": "1eist2",
        "ill-pl": "1eihin",
        "ill-pl-poss": "1eihi",
        "ade-pl": "1eill2",
        "abl-pl": "1eilt2",
        "all-pl": "1eille",
        "ess-pl": "1ein2",
        "tra-pl": "1eiksi",
        "tra-pl-poss": "1eikse",
        "ins-pl": "1ein",
        "ins-pl-poss": None,
        "abe-pl": "1eitt2",
        "cmt-pl": "1eine",
        "comp": "1impi",
        "sup": "1ein",
    },
    "fi-decl-ovi": {
    	"nargs": 4,
        "nom-sg": "12i",
        "nom-sg-poss": "12e",
        "gen-sg": "13en",
        "gen-sg-poss": "12e",
        "ptv-sg": "12e4",
        "ine-sg": "13ess4",
        "ela-sg": "13est4",
        "ill-sg": "12een",
        "ill-sg-poss": "12ee",
        "ade-sg": "13ell4",
        "abl-sg": "13elt4",
        "all-sg": "13elle",
        "ess-sg": "12en4",
        "tra-sg": "13eksi",
        "tra-sg-poss": "13ekse",
        "ins-sg": None,
        "abe-sg": "13ett4",
        "cmt-sg": None,
        "nom-pl": "13et",
        "nom-pl-poss": "12e",
        "gen-pl": "12ien",
        "gen-pl-poss": "12ie",
        "ptv-pl": "12i4",
        "ine-pl": "13iss4",
        "ela-pl": "13ist4",
        "ill-pl": "12iin",
        "ill-pl-poss": "12ii",
        "ade-pl": "13ill4",
        "abl-pl": "13ilt4",
        "all-pl": "13ille",
        "ess-pl": "12in4",
        "tra-pl": "13iksi",
        "tra-pl-poss": "13ikse",
        "ins-pl": "13in",
        "ins-pl-poss": None,
        "abe-pl": "13itt4",
        "cmt-pl": "12ine",
        "comp": "13empi",
        "sup": "13ein",
    },
    "fi-decl-nalle": {
    	"nargs": 4,
        "nom-sg": "12e",
        "gen-sg": "13en",
        "gen-sg-poss": "12e",
        "ptv-sg": "12e4",
        "ine-sg": "13ess4",
        "ela-sg": "13est4",
        "ill-sg": "12een",
        "ill-sg-poss": "12ee",
        "ade-sg": "13ell4",
        "abl-sg": "13elt4",
        "all-sg": "13elle",
        "ess-sg": "12en4",
        "tra-sg": "13eksi",
        "tra-sg-poss": "13ekse",
        "ins-sg": None,
        "abe-sg": "13ett4",
        "cmt-sg": None,
        "nom-pl": "13et",
        "nom-pl-poss": "12e",
        "gen-pl": ["12ejen", "12ein", "12ien"],  # latter two rare, last ?
        "gen-pl-poss": ["12eje", "12ei", "12ie"],  # last 2 rare, last ?
        "ptv-pl": "12ej4",
        "ine-pl": "13eiss4",
        "ela-pl": "13eist4",
        "ill-pl": "12eihin",
        "ill-pl-poss": "12eihi",
        "ade-pl": "13eill4",
        "abl-pl": "13eilt4",
        "all-pl": "13eille",
        "ess-pl": "12ein4",
        "tra-pl": "13eiksi",
        "tra-pl-poss": "13eikse",
        "ins-pl": "13ein",
        "ins-pl-poss": None,
        "abe-pl": "13eitt4",
        "cmt-pl": "12eine",
        "comp": "13empi",
        "sup": "13ein",
    },
    "fi-decl-kala": {
    	"nargs": 4,
        "nom-sg": "124",
        "gen-sg": "134n",
        "gen-sg-poss": "124",
        "ptv-sg": "1244",
        "ine-sg": "134ss4",
        "ela-sg": "134st4",
        "ill-sg": "124@n",
        "ill-sg-poss": "124@",
        "ade-sg": "134ll4",
        "abl-sg": "134lt4",
        "all-sg": "134lle",
        "ess-sg": "124n4",
        "tra-sg": "134ksi",
        "tra-sg-poss": "134kse",
        "ins-sg": None,
        "abe-sg": "134tt4",
        "cmt-sg": None,
        "nom-pl": "134t",
        "nom-pl-poss": "124",
        "gen-pl": ["124-Ojen", "124in"],  # Latter rare
        "gen-pl-poss": ["124-Oje", "124i"],  # Latter rare
        "ptv-pl": "124-Oj4",
        "ine-pl": "134-Oiss4",
        "ela-pl": "134-Oist4",
        "ill-pl": "124-Oihin",
        "ill-pl-poss": "124-Oihi",
        "ade-pl": "134-Oill4",
        "abl-pl": "134-Oilt4",
        "all-pl": "134-Oille",
        "ess-pl": "124-Oin4",
        "tra-pl": "134-Oiksi",
        "tra-pl-poss": "134-Oikse",
        "ins-pl": "134-Oin",
        "ins-pl-poss": None,
        "abe-pl": "134-Oitt4",
        "cmt-pl": "124-Oine",
        "comp": "134mpi",
        "sup": "134-Oin",
    },
    "fi-decl-koira": {
    	"nargs": 4,
        "nom-sg": "124",
        "gen-sg": "134n",
        "gen-sg-poss": "124",
        "ptv-sg": "1244",
        "ine-sg": "134ss4",
        "ela-sg": "134st4",
        "ill-sg": "124@n",
        "ill-sg-poss": "124@",
        "ade-sg": "134ll4",
        "abl-sg": "134lt4",
        "all-sg": "134lle",
        "ess-sg": "124n4",
        "tra-sg": "134ksi",
        "tra-sg-poss": "134kse",
        "ins-sg": None,
        "abe-sg": "134tt4",
        "cmt-sg": None,
        "nom-pl": "134t",
        "nom-pl-poss": "124",
        "gen-pl": ["12ien", "124in"],  # Latter rare
        "gen-pl-poss": ["12ie"],
        "ptv-pl": "12i4",
        "ine-pl": "13iss4",
        "ela-pl": "13ist4",
        "ill-pl": "12iin",
        "ill-pl-poss": "12ii",
        "ade-pl": "13ill4",
        "abl-pl": "13ilt4",
        "all-pl": "13ille",
        "ess-pl": "12in4",
        "tra-pl": "13iksi",
        "tra-pl-poss": "13ikse",
        "ins-pl": "13in",
        "ins-pl-poss": None,
        "abe-pl": "13itt4",
        "cmt-pl": "12ine",
        "comp": "134mpi",
        "sup": "13in",
    },
    "fi-decl-omena": {
    	"nargs": 2,
        "nom-sg": "12",
        "gen-sg": "12n",
        "gen-sg-poss": "12",
        "ptv-sg": "122",
        "ine-sg": "12ss2",
        "ela-sg": "12st2",
        "ill-sg": "12@n",
        "ill-sg-poss": "12@",
        "ade-sg": "12ll2",
        "abl-sg": "12lt2",
        "all-sg": "12lle",
        "ess-sg": "12n2",
        "tra-sg": "12ksi",
        "tra-sg-poss": "12kse",
        "ins-sg": None,
        "abe-sg": "12tt2",
        "cmt-sg": None,
        "nom-pl": "12t",
        "nom-pl-poss": "12",
        "gen-pl": ["1ien", "12-Oiden", "12-Oitten",
                   "12-Ojen", "12in"],  # Latter two rare
        "gen-pl-poss": ["1ie", "12-Oide", "12-Oitte", "12-Oje"],
        "ptv-pl": ["1i2", "12-Oit2", "12-Oj2"],
        "ine-pl": ["12-Oiss2", "1iss2"],
        "ela-pl": ["12-Oist2", "1ist2"],
        "ill-pl": ["12-Oihin", "1iin"],
        "ill-pl-poss": ["12-Oihi", "1ii"],
        "ade-pl": ["12-Oill2", "1ill2"],
        "abl-pl": ["12-Oilt2", "1ilt2"],
        "all-pl": ["12-Oille", "1ille"],
        "ess-pl": ["12-Oin2", "1in2"],
        "tra-pl": ["12-Oiksi", "1iksi"],
        "tra-pl-poss": ["12-Oikse", "1ikse"],
        "ins-pl": ["12-Oin", "1in"],
        "ins-pl-poss": None,
        "abe-pl": ["12-Oitt2", "1itt2"],
        "cmt-pl": ["12-Oine", "1ine"],
        "comp": "12mpi",
        "sup": "12-Oin",
    },
    "fi-decl-kulkija": {
    	"nargs": 2,
        "nom-sg": "12",
        "gen-sg": "12n",
        "gen-sg-poss": "12",
        "ptv-sg": "122",
        "ine-sg": "12ss2",
        "ela-sg": "12st2",
        "ill-sg": "12@n",
        "ill-sg-poss": "12@",
        "ade-sg": "12ll2",
        "abl-sg": "12lt2",
        "all-sg": "12lle",
        "ess-sg": "12n2",
        "tra-sg": "12ksi",
        "tra-sg-poss": "12kse",
        "ins-sg": None,
        "abe-sg": "12tt2",
        "cmt-sg": None,
        "nom-pl": "12t",
        "nom-pl-poss": "12",
        "gen-pl": ["12-Oiden", "12-Oitten", "12in"],  # Last rare
        "gen-pl-poss": ["12-Oide", "12-Oitte", "12i"],
        "ptv-pl": ["12-Oit2"],
        "ine-pl": ["12-Oiss2"],
        "ela-pl": ["12-Oist2"],
        "ill-pl": ["12-Oihin"],
        "ill-pl-poss": ["12-Oihi"],
        "ade-pl": ["12-Oill2"],
        "abl-pl": ["12-Oilt2"],
        "all-pl": ["12-Oille"],
        "ess-pl": ["12-Oin2"],
        "tra-pl": ["12-Oiksi"],
        "tra-pl-poss": ["12-Oikse"],
        "ins-pl": ["12-Oin"],
        "ins-pl-poss": None,
        "abe-pl": ["12-Oitt2"],
        "cmt-pl": ["12-Oine"],
        "comp": "12mpi",
        "sup": "12-Oin",
    },
    "fi-decl-katiska": {
    	"nargs": 2,
        "nom-sg": "12",
        "gen-sg": "12n",
        "gen-sg-poss": "12",
        "ptv-sg": "122",
        "ine-sg": "12ss2",
        "ela-sg": "12st2",
        "ill-sg": "12@n",
        "ill-sg-poss": "12@",
        "ade-sg": "12ll2",
        "abl-sg": "12lt2",
        "all-sg": "12lle",
        "ess-sg": "12n2",
        "tra-sg": "12ksi",
        "tra-sg-poss": "12kse",
        "ins-sg": None,
        "abe-sg": "12tt2",
        "cmt-sg": None,
        "nom-pl": "12t",
        "nom-pl-poss": "12",
        "gen-pl": ["12-Oiden", "12-Oitten",
                   "12-Ojen", "12in"],  # Last rare
        "gen-pl-poss": ["12-Oide", "12-Oitte",
                   "12-Oje", "12i"],  # Last rare
        "ptv-pl": ["12-Oit2", "12-Oj2"],
        "ine-pl": ["12-Oiss2"],
        "ela-pl": ["12-Oist2"],
        "ill-pl": ["12-Oihin"],
        "ill-pl-poss": ["12-Oihi"],
        "ade-pl": ["12-Oill2"],
        "abl-pl": ["12-Oilt2"],
        "all-pl": ["12-Oille"],
        "ess-pl": ["12-Oin2"],
        "tra-pl": ["12-Oiksi"],
        "tra-pl-poss": ["12-Oikse"],
        "ins-pl": ["12-Oin"],
        "ins-pl-poss": None,
        "abe-pl": ["12-Oitt2"],
        "cmt-pl": ["12-Oine"],
        "comp": "12mpi",
        "sup": "12-Oin",
    },
    "fi-decl-solakka": {
    	"nargs": 4,
        "nom-sg": "124",
        "gen-sg": "134n",
        "gen-sg-poss": "124",
        "ptv-sg": "1244",
        "ine-sg": "134ss4",
        "ela-sg": "134st4",
        "ill-sg": "124@n",
        "ill-sg-poss": "124@",
        "ade-sg": "134ll4",
        "abl-sg": "134lt4",
        "all-sg": "134lle",
        "ess-sg": "124n4",
        "tra-sg": "134ksi",
        "tra-sg-poss": "134kse",
        "ins-sg": None,
        "abe-sg": "134tt4",
        "cmt-sg": None,
        "nom-pl": "134t",
        "nom-pl-poss": "124",
        "gen-pl": ["134-Oiden", "134-Oitten", "124-Ojen",
                   "124in"],  # Last rare
        "gen-pl-poss": ["134-Oide", "134-Oitte", "124-Oje",
                        "124i"],  # Last rare
        "ptv-pl": ["134-Oit4", "124-Oj4"],
        "ine-pl": "134-Oiss4",
        "ela-pl": "134-Oist4",
        "ill-pl": ["134-Oihin", "124-Oihin"],
        "ill-pl-poss": ["134-Oihi", "124-Oihi"],
        "ade-pl": "134-Oill4",
        "abl-pl": "134-Oilt4",
        "all-pl": "134-Oille",
        "ess-pl": "124-Oin4",
        "tra-pl": "134-Oiksi",
        "tra-pl-poss": "134-Oikse",
        "ins-pl": "134-Oin",
        "ins-pl-poss": None,
        "abe-pl": "134-Oitt4",
        "cmt-pl": "124-Oine",
        "comp": "124mpi",
        "sup": "124-Oin",
    },
    "fi-decl-korkea": {
    	"nargs": 2,
        "nom-sg": "12",
        "gen-sg": "12n",
        "gen-sg-poss": "12",
        "ptv-sg": ["122", "12t2"],
        "ine-sg": "12ss2",
        "ela-sg": "12st2",
        "ill-sg": "12@n",
        "ill-sg-poss": "12@",
        "ade-sg": "12ll2",
        "abl-sg": "12lt2",
        "all-sg": "12lle",
        "ess-sg": "12n2",
        "tra-sg": "12ksi",
        "tra-sg-poss": "12kse",
        "ins-sg": None,
        "abe-sg": "12tt2",
        "cmt-sg": None,
        "nom-pl": "12t",
        "nom-pl-poss": "12",
        "gen-pl": ["1iden", "1itten", "12in"],  # Last rare
        "gen-pl-poss": ["1ide", "1itte", "12i"],  # Last rare
        "ptv-pl": ["1it2"],
        "ine-pl": ["1iss2"],
        "ela-pl": ["1ist2"],
        "ill-pl": ["1isiin", "1ihin"],
        "ill-pl-poss": ["1isii", "1ihi"],
        "ade-pl": ["1ill2"],
        "abl-pl": ["1ilt2"],
        "all-pl": ["1ille"],
        "ess-pl": ["1in2"],
        "tra-pl": ["1iksi"],
        "tra-pl-poss": ["1ikse"],
        "ins-pl": ["1in"],
        "ins-pl-poss": None,
        "abe-pl": ["1itt2"],
        "cmt-pl": ["1ine"],
        "comp": "12mpi",
        "sup": "1in",
    },
    "fi-decl-vanhempi": {
    	"nargs": 2,
        "nom-sg": "1mpi",
        "nom-sg-poss": "1mp2",
        "gen-sg": "1mm2n",
        "gen-sg-poss": "1mp2",
        "ptv-sg": "1mp22",
        "ine-sg": "1mm2ss2",
        "ela-sg": "1mm2st2",
        "ill-sg": "1mp2@n",
        "ill-sg-poss": "1mp2@",
        "ade-sg": "1mm2ll2",
        "abl-sg": "1mm2lt2",
        "all-sg": "1mm2lle",
        "ess-sg": "1mp2n2",
        "tra-sg": "1mm2ksi",
        "tra-sg-poss": "1mm2kse",
        "ins-sg": None,
        "abe-sg": "1mm2tt2",
        "cmt-sg": None,
        "nom-pl": "1mm2t",
        "nom-pl-poss": "1mp2",
        "gen-pl": ["1mpien", "1mp2in"],
        "gen-pl-poss": ["1mpie", "1mp2i"],
        "ptv-pl": ["1mpi2"],
        "ine-pl": "1mmiss2",
        "ela-pl": "1mmist2",
        "ill-pl": "1mpiin",
        "ill-pl-poss": "1mpii",
        "ade-pl": "1mmill2",
        "abl-pl": "1mmilt2",
        "all-pl": "1mmille",
        "ess-pl": "1mpin2",
        "tra-pl": "1mmiksi",
        "tra-pl-poss": "1mmikse",
        "ins-pl": "1mmin",
        "ins-pl-poss": None,
        "abe-pl": "1mmitt2",
        "cmt-pl": "1mpine",
    },
    "fi-decl-vapaa": {
        # Note: also includes words like "takuu"
    	"nargs": 2,
        "nom-sg": "1@",
        "gen-sg": "1@n",
        "gen-sg-poss": "1@",
        "ptv-sg": ["1@t2"],
        "ine-sg": "1@ss2",
        "ela-sg": "1@st2",
        "ill-sg": "1@seen",
        "ill-sg-poss": "1@see",
        "ade-sg": "1@ll2",
        "abl-sg": "1@lt2",
        "all-sg": "1@lle",
        "ess-sg": "1@n2",
        "tra-sg": "1@ksi",
        "tra-sg-poss": "1@kse",
        "ins-sg": None,
        "abe-sg": "1@tt2",
        "cmt-sg": None,
        "nom-pl": "1@t",
        "nom-pl-poss": "1@",
        "gen-pl": ["1iden", "1itten"],
        "gen-pl-poss": ["1ide", "1itte"],
        "ptv-pl": ["1it2"],
        "ine-pl": ["1iss2"],
        "ela-pl": ["1ist2"],
        "ill-pl": ["1isiin", "1ihin"],  # Last rare
        "ill-pl-poss": ["1isii", "1ihi"],  # Last rare
        "ade-pl": ["1ill2"],
        "abl-pl": ["1ilt2"],
        "all-pl": ["1ille"],
        "ess-pl": ["1in2"],
        "tra-pl": ["1iksi"],
        "tra-pl-poss": ["1ikse"],
        "ins-pl": ["1in"],
        "ins-pl-poss": None,
        "abe-pl": ["1itt2"],
        "cmt-pl": ["1ine"],
        "comp": "1@mpi",
        "sup": "1in",
    },
    "fi-decl-maa": {
    	"nargs": 2,
        "nom-sg": "1",
        "gen-sg": "1n",
        "gen-sg-poss": "1",
        "ptv-sg": ["1t2"],
        "ine-sg": "1ss2",
        "ela-sg": "1st2",
        "ill-sg": "1h@n",
        "ill-sg-poss": "1h@",
        "ade-sg": "1ll2",
        "abl-sg": "1lt2",
        "all-sg": "1lle",
        "ess-sg": "1n2",
        "tra-sg": "1ksi",
        "tra-sg-poss": "1kse",
        "ins-sg": None,
        "abe-sg": "1tt2",
        "cmt-sg": None,
        "nom-pl": "1t",
        "nom-pl-poss": "1",
        "gen-pl": ["1-iden", "1-itten"],
        "gen-pl-poss": ["1-ide", "1-itte"],
        "ptv-pl": ["1-it2"],
        "ine-pl": ["1-iss2"],
        "ela-pl": ["1-ist2"],
        "ill-pl": ["1-ihin"],
        "ill-pl-poss": ["1-ihi"],
        "ade-pl": ["1-ill2"],
        "abl-pl": ["1-ilt2"],
        "all-pl": ["1-ille"],
        "ess-pl": ["1-in2"],
        "tra-pl": ["1-iksi"],
        "tra-pl-poss": ["1-ikse"],
        "ins-pl": ["1-in"],
        "ins-pl-poss": None,
        "abe-pl": ["1-itt2"],
        "cmt-pl": ["1-ine"],
        "comp": "1mpi",
        "sup": "1-in",
    },
    "fi-decl-suo": {
    	"nargs": 2,
        "nom-sg": "1",
        "gen-sg": "1n",
        "gen-sg-poss": "1",
        "ptv-sg": ["1t2"],
        "ine-sg": "1ss2",
        "ela-sg": "1st2",
        "ill-sg": "1h@n",
        "ill-sg-poss": "1h@",
        "ade-sg": "1ll2",
        "abl-sg": "1lt2",
        "all-sg": "1lle",
        "ess-sg": "1n2",
        "tra-sg": "1ksi",
        "tra-sg-poss": "1kse",
        "ins-sg": None,
        "abe-sg": "1tt2",
        "cmt-sg": None,
        "nom-pl": "1t",
        "nom-pl-poss": "1",
        "gen-pl": ["1/iden", "1/itten"],
        "gen-pl-poss": ["1/ide", "1/itte"],
        "ptv-pl": ["1/it2"],
        "ine-pl": ["1/iss2"],
        "ela-pl": ["1/ist2"],
        "ill-pl": ["1/ihin"],
        "ill-pl-poss": ["1/ihi"],
        "ade-pl": ["1/ill2"],
        "abl-pl": ["1/ilt2"],
        "all-pl": ["1/ille"],
        "ess-pl": ["1/in2"],
        "tra-pl": ["1/iksi"],
        "tra-pl-poss": ["1/ikse"],
        "ins-pl": ["1/in"],
        "ins-pl-poss": None,
        "abe-pl": ["1/itt2"],
        "cmt-pl": ["1/ine"],
        "comp": "1mpi",
        "sup": "1/in",
    },
    "fi-decl-rosé": {
    	"nargs": 2,
        "nom-sg": "1",
        "gen-sg": "1n",
        "gen-sg-poss": "1",
        "ptv-sg": ["1t2"],
        "ine-sg": "1ss2",
        "ela-sg": "1st2",
        "ill-sg": "1h@n",
        "ill-sg-poss": "1h@",
        "ade-sg": "1ll2",
        "abl-sg": "1lt2",
        "all-sg": "1lle",
        "ess-sg": "1n2",
        "tra-sg": "1ksi",
        "tra-sg-poss": "1kse",
        "ins-sg": None,
        "abe-sg": "1tt2",
        "cmt-sg": None,
        "nom-pl": "1t",
        "nom-pl-poss": "1",
        "gen-pl": ["1iden", "1itten"],
        "gen-pl-poss": ["1ide", "1itte"],
        "ptv-pl": ["1it2"],
        "ine-pl": ["1iss2"],
        "ela-pl": ["1ist2"],
        "ill-pl": ["1ihin"],
        "ill-pl-poss": ["1ihi"],
        "ade-pl": ["1ill2"],
        "abl-pl": ["1ilt2"],
        "all-pl": ["1ille"],
        "ess-pl": ["1in2"],
        "tra-pl": ["1iksi"],
        "tra-pl-poss": ["1ikse"],
        "ins-pl": ["1in"],
        "ins-pl-poss": None,
        "abe-pl": ["1itt2"],
        "cmt-pl": ["1ine"],
        "comp": "1mpi",
        "sup": "1in",
    },
    "fi-decl-parfait": {
    	"nargs": 2,
        "nom-sg": "1",
        "nom-sg-clitic": "1'",
        "nom-sg-poss": "1'",
        "gen-sg": "1'n",
        "gen-sg-poss": "1'",
        "ptv-sg": ["1't2"],
        "ine-sg": "1'ss2",
        "ela-sg": "1'st2",
        "ill-sg": "1'h@n",
        "ill-sg-poss": "1'h@",
        "ade-sg": "1'll2",
        "abl-sg": "1'lt2",
        "all-sg": "1'lle",
        "ess-sg": "1'n2",
        "tra-sg": "1'ksi",
        "tra-sg-poss": "1'kse",
        "ins-sg": None,
        "abe-sg": "1'tt2",
        "cmt-sg": None,
        "nom-pl": "1't",
        "nom-pl-poss": "1'",
        "gen-pl": ["1'iden", "1'itten"],
        "gen-pl-poss": ["1'ide", "1'itte"],
        "ptv-pl": ["1'it2"],
        "ine-pl": ["1'iss2"],
        "ela-pl": ["1'ist2"],
        "ill-pl": ["1'ihin"],
        "ill-pl-poss": ["1'ihi"],
        "ade-pl": ["1'ill2"],
        "abl-pl": ["1'ilt2"],
        "all-pl": ["1'ille"],
        "ess-pl": ["1'in2"],
        "tra-pl": ["1'iksi"],
        "tra-pl-poss": ["1'ikse"],
        "ins-pl": ["1'in"],
        "ins-pl-poss": None,
        "abe-pl": ["1'itt2"],
        "cmt-pl": ["1'ine"],
        "comp": "1'mpi",
        "sup": "1'in",
    },
    "fi-decl-tiili": {
    	"nargs": 2,
        "nom-sg": "1i",
        "nom-sg-poss": "1e",
        "gen-sg": "1en",
        "gen-sg-poss": "1e",
        "ptv-sg": "1t2",
        "ine-sg": "1ess2",
        "ela-sg": "1est2",
        "ill-sg": "1een",
        "ill-sg-poss": "1ee",
        "ade-sg": "1ell2",
        "abl-sg": "1elt2",
        "all-sg": "1elle",
        "ess-sg": "1en2",
        "tra-sg": "1eksi",
        "tra-sg-poss": "1ekse",
        "ins-sg": None,
        "abe-sg": "1ett2",
        "cmt-sg": None,
        "nom-pl": "1et",
        "nom-pl-poss": "1e",
        "gen-pl": "1ien",
        "gen-pl-poss": "1ie",
        "ptv-pl": "1i2",
        "ine-pl": "1iss2",
        "ela-pl": "1ist2",
        "ill-pl": "1iin",
        "ill-pl-poss": "1ii",
        "ade-pl": "1ill2",
        "abl-pl": "1ilt2",
        "all-pl": "1ille",
        "ess-pl": "1in2",
        "tra-pl": "1iksi",
        "tra-pl-poss": "1ikse",
        "ins-pl": "1in",
        "ins-pl-poss": None,
        "abe-pl": "1itt2",
        "cmt-pl": "1ine",
        "comp": "1empi",
        "sup": "1ein",
    },
    "fi-decl-uni": {
    	"nargs": 2,
        "nom-sg": "1i",
        "nom-sg-poss": "1e",
        "gen-sg": "1en",
        "gen-sg-poss": "1e",
        "ptv-sg": "1t2-9",
        "ine-sg": "1ess2",
        "ela-sg": "1est2",
        "ill-sg": "1een",
        "ill-sg-poss": "1ee",
        "ade-sg": "1ell2",
        "abl-sg": "1elt2",
        "all-sg": "1elle",
        "ess-sg": "1en2",
        "tra-sg": "1eksi",
        "tra-sg-poss": "1ekse",
        "ins-sg": None,
        "abe-sg": "1ett2",
        "cmt-sg": None,
        "nom-pl": "1et",
        "nom-pl-poss": "1e",
        "gen-pl": ["1ien", "1ten"],
        "gen-pl-poss": ["1ie", "1te"],
        "ptv-pl": "1i2",
        "ine-pl": "1iss2",
        "ela-pl": "1ist2",
        "ill-pl": "1iin",
        "ill-pl-poss": "1ii",
        "ade-pl": "1ill2",
        "abl-pl": "1ilt2",
        "all-pl": "1ille",
        "ess-pl": "1in2",
        "tra-pl": "1iksi",
        "tra-pl-poss": "1ikse",
        "ins-pl": "1in",
        "ins-pl-poss": None,
        "abe-pl": "1itt2",
        "cmt-pl": "1ine",
        "comp": "1impi",
        "sup": "1in",
    },
    "fi-decl-toimi": {
    	"nargs": 2,
        "nom-sg": "1mi",
        "nom-sg-poss": "1me",
        "gen-sg": "1men",
        "gen-sg-poss": "1me",
        "ptv-sg": ["1nt2", "1me2"],
        "ine-sg": "1mess2",
        "ela-sg": "1mest2",
        "ill-sg": "1meen",
        "ill-sg-poss": "1mee",
        "ade-sg": "1mell2",
        "abl-sg": "1melt2",
        "all-sg": "1melle",
        "ess-sg": "1men2",
        "tra-sg": "1meksi",
        "tra-sg-poss": "1mekse",
        "ins-sg": None,
        "abe-sg": "1mett2",
        "cmt-sg": None,
        "nom-pl": "1met",
        "nom-pl-poss": "1me",
        "gen-pl": ["1mien", "1nten"],
        "gen-pl-poss": ["1mie", "1nte"],
        "ptv-pl": "1mi2",
        "ine-pl": "1miss2",
        "ela-pl": "1mist2",
        "ill-pl": "1miin",
        "ill-pl-poss": "1mii",
        "ade-pl": "1mill2",
        "abl-pl": "1milt2",
        "all-pl": "1mille",
        "ess-pl": "1min2",
        "tra-pl": "1miksi",
        "tra-pl-poss": "1mikse",
        "ins-pl": "1min",
        "ins-pl-poss": None,
        "abe-pl": "1mitt2",
        "cmt-pl": "1mine",
        "comp": "1mempi",
        "sup": "1mein",
    },
    "fi-decl-pieni": {
    	"nargs": 2,
        "nom-sg": "1i",
        "nom-sg-poss": "1e",
        "gen-sg": "1en",
        "gen-sg-poss": "1e",
        "ptv-sg": "1t2-9",
        "ine-sg": "1ess2",
        "ela-sg": "1est2",
        "ill-sg": "1een",
        "ill-sg-poss": "1ee",
        "ade-sg": "1ell2",
        "abl-sg": "1elt2",
        "all-sg": "1elle",
        "ess-sg": "1en2",
        "tra-sg": "1eksi",
        "tra-sg-poss": "1ekse",
        "ins-sg": None,
        "abe-sg": "1ett2",
        "cmt-sg": None,
        "nom-pl": "1et",
        "nom-pl-poss": "1e",
        "gen-pl": ["1ten", "1ien"],
        "gen-pl-poss": ["1te", "1ie"],
        "ptv-pl": "1i2",
        "ine-pl": "1iss2",
        "ela-pl": "1ist2",
        "ill-pl": "1iin",
        "ill-pl-poss": "1ii",
        "ade-pl": "1ill2",
        "abl-pl": "1ilt2",
        "all-pl": "1ille",
        "ess-pl": "1in2",
        "tra-pl": "1iksi",
        "tra-pl-poss": "1ikse",
        "ins-pl": "1in",
        "ins-pl-poss": None,
        "abe-pl": "1itt2",
        "cmt-pl": "1ine",
        "comp": "1empi",
        "sup": "1in",
    },
    "fi-decl-käsi": {
    	"nargs": 2,
        "nom-sg": "1si",
        "nom-sg-poss": "1te",
        "gen-sg": "1den",
        "gen-sg-poss": "1te",
        "ptv-sg": "1tt2",
        "ine-sg": "1dess2",
        "ela-sg": "1dest2",
        "ill-sg": "1teen",
        "ill-sg-poss": "1tee",
        "ade-sg": "1dell2",
        "abl-sg": "1delt2",
        "all-sg": "1delle",
        "ess-sg": "1ten2",
        "tra-sg": "1deksi",
        "tra-sg-poss": "1dekse",
        "ins-sg": None,
        "abe-sg": "1dett2",
        "cmt-sg": None,
        "nom-pl": "1det",
        "nom-pl-poss": "1te",
        "gen-pl": ["1sien", "1tten"],  # Latter rare
        "gen-pl-poss": ["1sie", "1tte"],  # Latter rare
        "ptv-pl": "1si2",
        "ine-pl": "1siss2",
        "ela-pl": "1sist2",
        "ill-pl": "1siin",
        "ill-pl-poss": "1sii",
        "ade-pl": "1sill2",
        "abl-pl": "1silt2",
        "all-pl": "1sille",
        "ess-pl": "1sin2",
        "tra-pl": "1siksi",
        "tra-pl-poss": "1sikse",
        "ins-pl": "1sin",
        "ins-pl-poss": None,
        "abe-pl": "1sitt2",
        "cmt-pl": "1sine",
        "comp": "1sempi",
        "sup": "1sin",
    },
    "fi-decl-kynsi": {
    	"nargs": 2,
        "nom-sg": "1si",
        "nom-sg-poss": "1te",
        "gen-sg": "1nen",
        "gen-sg-poss": "1te",
        "ptv-sg": "1tt2",
        "ine-sg": "1ness2",
        "ela-sg": "1nest2",
        "ill-sg": "1teen",
        "ill-sg-poss": "1tee",
        "ade-sg": "1nell2",
        "abl-sg": "1nelt2",
        "all-sg": "1nelle",
        "ess-sg": "1ten2",
        "tra-sg": "1neksi",
        "tra-sg-poss": "1nekse",
        "ins-sg": None,
        "abe-sg": "1nett2",
        "cmt-sg": None,
        "nom-pl": "1net",
        "nom-pl-poss": "1te",
        "gen-pl": ["1sien", "1tten"],  # Latter rare
        "gen-pl-poss": ["1sie", "1tte"],  # Latter rare
        "ptv-pl": "1si2",
        "ine-pl": "1siss2",
        "ela-pl": "1sist2",
        "ill-pl": "1siin",
        "ill-pl-poss": "1sii",
        "ade-pl": "1sill2",
        "abl-pl": "1silt2",
        "all-pl": "1sille",
        "ess-pl": "1sin2",
        "tra-pl": "1siksi",
        "tra-pl-poss": "1sikse",
        "ins-pl": "1sin",
        "ins-pl-poss": None,
        "abe-pl": "1sitt2",
        "cmt-pl": "1sine",
        "comp": "1nempi",
        "sup": "1sin",
    },
    "fi-decl-lapsi": {
    	"nargs": 2,
        "nom-sg": "1si",
        "nom-sg-poss": "1se",
        "gen-sg": "1sen",
        "gen-sg-poss": "1se",
        "ptv-sg": "1-st2",
        "ine-sg": "1sess2",
        "ela-sg": "1sest2",
        "ill-sg": "1seen",
        "ill-sg-poss": "1see",
        "ade-sg": "1sell2",
        "abl-sg": "1selt2",
        "all-sg": "1selle",
        "ess-sg": "1sen2",
        "tra-sg": "1seksi",
        "tra-sg-poss": "1sekse",
        "ins-sg": None,
        "abe-sg": "1sett2",
        "cmt-sg": None,
        "nom-pl": "1set",
        "nom-pl-poss": "1se",
        "gen-pl": ["1sien", "1-sten"],
        "gen-pl-poss": ["1sie", "1-ste"],
        "ptv-pl": "1si2",
        "ine-pl": "1siss2",
        "ela-pl": "1sist2",
        "ill-pl": "1siin",
        "ill-pl-poss": "1sii",
        "ade-pl": "1sill2",
        "abl-pl": "1silt2",
        "all-pl": "1sille",
        "ess-pl": "1sin2",
        "tra-pl": "1siksi",
        "tra-pl-poss": "1sikse",
        "ins-pl": "1sin",
        "ins-pl-poss": None,
        "abe-pl": "1sitt2",
        "cmt-pl": "1sine",
        "comp": "1sempi",
        "sup": "1sin",
    },
    "fi-decl-veitsi": {
    	"nargs": 2,
        "nom-sg": "1tsi",
        "nom-sg-poss": "1tse",
        "gen-sg": "1tsen",
        "gen-sg-poss": "1tse",
        "ptv-sg": "1st2",
        "ine-sg": "1tsess2",
        "ela-sg": "1tsest2",
        "ill-sg": "1tseen",
        "ill-sg-poss": "1tsee",
        "ade-sg": "1tsell2",
        "abl-sg": "1tselt2",
        "all-sg": "1tselle",
        "ess-sg": "1tsen2",
        "tra-sg": "1tseksi",
        "tra-sg-poss": "1tsekse",
        "ins-sg": None,
        "abe-sg": "1tsett2",
        "cmt-sg": None,
        "nom-pl": "1tset",
        "nom-pl-poss": "1tse",
        "gen-pl": ["1tsien", "1sten"],
        "gen-pl-poss": ["1tsie", "1ste"],
        "ptv-pl": "1tsi2",
        "ine-pl": "1tsiss2",
        "ela-pl": "1tsist2",
        "ill-pl": "1tsiin",
        "ill-pl-poss": "1tsii",
        "ade-pl": "1tsill2",
        "abl-pl": "1tsilt2",
        "all-pl": "1tsille",
        "ess-pl": "1tsin2",
        "tra-pl": "1tsiksi",
        "tra-pl-poss": "1tsikse",
        "ins-pl": "1tsin",
        "ins-pl-poss": None,
        "abe-pl": "1tsitt2",
        "cmt-pl": "1tsine",
        "comp": "1tsempi",
        "sup": "1tsin",
    },
    "fi-decl-kaksi": {
    	"nargs": 2,
        "nom-sg": "1ksi",
        "nom-sg-poss": "1hte",
        "gen-sg": "1hden",
        "gen-sg-poss": "1hte",
        "ptv-sg": "1ht2",
        "ine-sg": "1hdess2",
        "ela-sg": "1hdest2",
        "ill-sg": "1hteen",
        "ill-sg-poss": "1htee",
        "ade-sg": "1hdell2",
        "abl-sg": "1hdelt2",
        "all-sg": "1hdelle",
        "ess-sg": "1hten2",
        "tra-sg": "1hdeksi",
        "tra-sg-poss": "1hdekse",
        "ins-sg": None,
        "abe-sg": "1hdett2",
        "cmt-sg": None,
        "nom-pl": "1hdet",
        "nom-pl-poss": "1hte",
        "gen-pl": ["1ksien"],
        "gen-pl-poss": ["1ksie"],
        "ptv-pl": "1ksi2",
        "ine-pl": "1ksiss2",
        "ela-pl": "1ksist2",
        "ill-pl": "1ksiin",
        "ill-pl-poss": "1ksii",
        "ade-pl": "1ksill2",
        "abl-pl": "1ksilt2",
        "all-pl": "1ksille",
        "ess-pl": "1ksin2",
        "tra-pl": "1ksiksi",
        "tra-pl-poss": "1ksikse",
        "ins-pl": "1ksin",
        "ins-pl-poss": None,
        "abe-pl": "1ksitt2",
        "cmt-pl": "1ksine",
        "comp": "1ksimpi",
        "sup": "1ksin",
    },
    "fi-decl-sisar": {
    	"nargs": 5,
        "nom-sg": "124",
        "nom-sg-poss": "134e",
        "gen-sg": "134en",
        "gen-sg-poss": "134e",
        "ptv-sg": "124t5",
        "ine-sg": "134ess5",
        "ela-sg": "134est5",
        "ill-sg": "134een",
        "ill-sg-poss": "134ee",
        "ade-sg": "134ell5",
        "abl-sg": "134elt5",
        "all-sg": "134elle",
        "ess-sg": "134en5",
        "tra-sg": "134eksi",
        "tra-sg-poss": "134ekse",
        "ins-sg": None,
        "abe-sg": "134ett5",
        "cmt-sg": None,
        "nom-pl": "134et",
        "nom-pl-poss": "134e",
        "gen-pl": ["134ien", "124ten"],
        "gen-pl-poss": ["134ie", "124te"],
        "ptv-pl": "134i5",
        "ine-pl": "134iss5",
        "ela-pl": "134ist5",
        "ill-pl": "134iin",
        "ill-pl-poss": "134ii",
        "ade-pl": "134ill5",
        "abl-pl": "134ilt5",
        "all-pl": "134ille",
        "ess-pl": "134in5",
        "tra-pl": "134iksi",
        "tra-pl-poss": "134ikse",
        "ins-pl": "134in",
        "ins-pl-poss": None,
        "abe-pl": "134itt5",
        "cmt-pl": "134ine",
        "comp": "134empi",
        "sup": "134in",
    },
    "fi-decl-kytkin": {
    	"nargs": 5,
        "nom-sg": "124n",
        "nom-sg-poss": "134me",
        "gen-sg": "134men",
        "gen-sg-poss": "134me",
        "ptv-sg": "124nt5",
        "ine-sg": "134mess5",
        "ela-sg": "134mest5",
        "ill-sg": "134meen",
        "ill-sg-poss": "134mee",
        "ade-sg": "134mell5",
        "abl-sg": "134melt5",
        "all-sg": "134melle",
        "ess-sg": "134men5",
        "tra-sg": "134meksi",
        "tra-sg-poss": "134mekse",
        "ins-sg": None,
        "abe-sg": "134mett5",
        "cmt-sg": None,
        "nom-pl": "134met",
        "nom-pl-poss": "134me",
        "gen-pl": ["134mien", "124nten"],
        "gen-pl-poss": ["134mie", "124nte"],
        "ptv-pl": "134mi5",
        "ine-pl": "134miss5",
        "ela-pl": "134mist5",
        "ill-pl": "134miin",
        "ill-pl-poss": "134mii",
        "ade-pl": "134mill5",
        "abl-pl": "134milt5",
        "all-pl": "134mille",
        "ess-pl": "134min5",
        "tra-pl": "134miksi",
        "tra-pl-poss": "134mikse",
        "ins-pl": "134min",
        "ins-pl-poss": None,
        "abe-pl": "134mitt5",
        "cmt-pl": "134mine",
        "comp": "134mempi",
        "sup": "134min",
    },
    "fi-decl-onneton": {
    	"nargs": 2,
        "nom-sg": "12-tOn",
        "nom-sg-poss": "12-ttOm2",
        "gen-sg": "12-ttOm2n",
        "gen-sg-poss": "12-ttOm2",
        "ptv-sg": "12-tOnt2",
        "ine-sg": "12-ttOm2ss2",
        "ela-sg": "12-ttOm2st2",
        "ill-sg": "12-ttOm2@n",
        "ill-sg-poss": "12-ttOm2@",
        "ade-sg": "12-ttOm2ll2",
        "abl-sg": "12-ttOm2lt2",
        "all-sg": "12-ttOm2lle",
        "ess-sg": "12-ttOm2n2",
        "tra-sg": "12-ttOm2ksi",
        "tra-sg-poss": "12-ttOm2kse",
        "ins-sg": None,
        "abe-sg": "12-ttOm2tt2",
        "cmt-sg": None,
        "nom-pl": "12-ttOm2t",
        "nom-pl-poss": "12-ttOm2",
        "gen-pl": ["12-ttOmien", "12-tOnten"],  # Latter rare
        "gen-pl-poss": ["12-ttOmie", "12-tOnte"],  # Latter rare
        "ptv-pl": "12-ttOmi2",
        "ine-pl": "12-ttOmiss2",
        "ela-pl": "12-ttOmist2",
        "ill-pl": "12-ttOmiin",
        "ill-pl-poss": "12-ttOmii",
        "ade-pl": "12-ttOmill2",
        "abl-pl": "12-ttOmilt2",
        "all-pl": "12-ttOmille",
        "ess-pl": "12-ttOmin2",
        "tra-pl": "12-ttOmiksi",
        "tra-pl-poss": "12-ttOmikse",
        "ins-pl": "12-ttOmin",
        "ins-pl-poss": None,
        "abe-pl": "12-ttOmitt2",
        "cmt-pl": "12-ttOmine",
        "comp": "12-ttOm2mpi",
        "sup": "12-ttOmin",
    },
    "fi-decl-lämmin": {
    	"nargs": 0,
        "nom-sg": "1lämmin",
        "nom-sg-poss": "1lämpimä",
        "gen-sg": "1lämpimän",
        "gen-sg-poss": "1lämpimä",
        "ptv-sg": "1lämmintä",
        "ine-sg": "1lämpimässä",
        "ela-sg": "1lämpimästä",
        "ill-sg": "1lämpimään",
        "ill-sg-poss": "1lämpimää",
        "ade-sg": "1lämpimällä",
        "abl-sg": "1lämpimältä",
        "all-sg": "1lämpimälle",
        "ess-sg": "1lämpimänä",
        "tra-sg": "1lämpimäksi",
        "tra-sg-poss": "1lämpimäkse",
        "ins-sg": None,
        "abe-sg": "1lämpimättä",
        "cmt-sg": None,
        "nom-pl": "1lämpimät",
        "nom-pl-poss": "1lämpimä",
        "gen-pl": ["1lämpimien", "1lämminten"],
        "gen-pl-poss": ["1lämpimie", "1lämminte"],
        "ptv-pl": "1lämpimiä",
        "ine-pl": "1lämpimissä",
        "ela-pl": "1lämpimistä",
        "ill-pl": "1lämpimiin",
        "ill-pl-poss": "1lämpimii",
        "ade-pl": "1lämpimillä",
        "abl-pl": "1lämpimiltä",
        "all-pl": "1lämpimille",
        "ess-pl": "1lämpiminä",
        "tra-pl": "1lämpimiksi",
        "tra-pl-poss": "1lämpimikse",
        "ins-pl": "1lämpimin",
        "ins-pl-poss": None,
        "abe-pl": "1lämpimittä",
        "cmt-pl": "1lämpimine",
        "comp": "1lämpimämpi",
        "sup": "1lämpimin",
    },
    "fi-decl-sisin": {
    	"nargs": 2,
        "nom-sg": "1in",
        "nom-sg-poss": "1imp2",
        "gen-sg": "1imm2n",
        "gen-sg-poss": "1imp2",
        "ptv-sg": "1int2",
        "ine-sg": "1imm2ss2",
        "ela-sg": "1imm2st2",
        "ill-sg": "1imp2@n",
        "ill-sg-poss": "1imp2@",
        "ade-sg": "1imm2ll2",
        "abl-sg": "1imm2lt2",
        "all-sg": "1imm2lle",
        "ess-sg": "1imp2n2",
        "tra-sg": "1imm2ksi",
        "tra-sg-poss": "1imm2kse",
        "ins-sg": None,
        "abe-sg": "1imm2tt2",
        "cmt-sg": None,
        "nom-pl": "1imm2t",
        "nom-pl-poss": "1imp2",
        "gen-pl": ["1impien", "1inten"],
        "gen-pl-poss": ["1impie", "1inte"],
        "ptv-pl": ["1impi2"],
        "ine-pl": "1immiss2",
        "ela-pl": "1immist2",
        "ill-pl": "1impiin",
        "ill-pl-poss": "1impii",
        "ade-pl": "1immill2",
        "abl-pl": "1immilt2",
        "all-pl": "1immille",
        "ess-pl": "1impin2",
        "tra-pl": "1immiksi",
        "tra-pl-poss": "1immikse",
        "ins-pl": "1immin",
        "ins-pl-poss": None,
        "abe-pl": "1immitt2",
        "cmt-pl": "1impine",
    },
    "fi-decl-vasen": {
    	"nargs": 0,
        "nom-sg": "1vasen",
        "nom-sg-poss": "1vasempa",
        "gen-sg": "1vasemman",
        "gen-sg-poss": "1vasempa",
        "ptv-sg": ["1vasenta", "vasempaa"],
        "ine-sg": "1vasemmassa",
        "ela-sg": "1vasemmasta",
        "ill-sg": "1vasempaan",
        "ill-sg-poss": "1vasempaa",
        "ade-sg": "1vasemmalla",
        "abl-sg": "1vasemmalta",
        "all-sg": "1vasemmalle",
        "ess-sg": "1vasempana",
        "tra-sg": "1vasemmaksi",
        "tra-sg-poss": "1vasemmakse",
        "ins-sg": None,
        "abe-sg": "1vasemmatta",
        "cmt-sg": None,
        "nom-pl": "1vasemmat",
        "nom-pl-poss": "1vasempa",
        "gen-pl": ["1vasempien", "1vasenten", "vasempain"],  # Last rare
        "gen-pl-poss": ["1vasempie", "1vasente", "vasempai"],  # Last rare
        "ptv-pl": "1vasempia",
        "ine-pl": "1vasemmissa",
        "ela-pl": "1vasemmista",
        "ill-pl": "1vasempiin",
        "ill-pl-poss": "1vasempii",
        "ade-pl": "1vasemmilla",
        "abl-pl": "1vasemmilta",
        "all-pl": "1vasemmille",
        "ess-pl": "1vasempina",
        "tra-pl": "1vasemmiksi",
        "tra-pl-poss": "1vasemmikse",
        "ins-pl": "1vasemmin",
        "ins-pl-poss": None,
        "abe-pl": "1vasemmitta",
        "cmt-pl": "1vasempine",
        "comp": ["1vasempi", "1vasemmanpuoleisempi"],
        "sup": "1vasemmanpuoleisin",
    },
    "fi-decl-nainen": {
    	"nargs": 2,
        "nom-sg": "1nen",
        "nom-sg-poss": "1se",
        "gen-sg": "1sen",
        "gen-sg-poss": "1se",
        "ptv-sg": "1st2",
        "ine-sg": "1sess2",
        "ela-sg": "1sest2",
        "ill-sg": "1seen",
        "ill-sg-poss": "1see",
        "ade-sg": "1sell2",
        "abl-sg": "1selt2",
        "all-sg": "1selle",
        "ess-sg": "1sen2",
        "tra-sg": "1seksi",
        "tra-sg-poss": "1sekse",
        "ins-sg": None,
        "abe-sg": "1sett2",
        "cmt-sg": None,
        "nom-pl": "1set",
        "nom-pl-poss": "1se",
        "gen-pl": ["1sten", "1sien"],
        "gen-pl-poss": ["1ste", "1sie"],
        "ptv-pl": "1si2",
        "ine-pl": "1siss2",
        "ela-pl": "1sist2",
        "ill-pl": "1siin",
        "ill-pl-poss": "1sii",
        "ade-pl": "1sill2",
        "abl-pl": "1silt2",
        "all-pl": "1sille",
        "ess-pl": "1sin2",
        "tra-pl": "1siksi",
        "tra-pl-poss": "1sikse",
        "ins-pl": "1sin",
        "ins-pl-poss": None,
        "abe-pl": "1sitt2",
        "cmt-pl": "1sine",
        "comp": "1sempi",
        "sup": "1sin",
    },
    "fi-decl-vastaus": {
    	"nargs": 2,
        "nom-sg": "1s",
        "nom-sg-poss": "1kse",
        "gen-sg": "1ksen",
        "gen-sg-poss": "1kse",
        "ptv-sg": "1st2",
        "ine-sg": "1ksess2",
        "ela-sg": "1ksest2",
        "ill-sg": "1kseen",
        "ill-sg-poss": "1ksee",
        "ade-sg": "1ksell2",
        "abl-sg": "1kselt2",
        "all-sg": "1kselle",
        "ess-sg": "1ksen2",
        "tra-sg": "1kseksi",
        "tra-sg-poss": "1ksekse",
        "ins-sg": None,
        "abe-sg": "1ksett2",
        "cmt-sg": None,
        "nom-pl": "1kset",
        "nom-pl-poss": "1kse",
        "gen-pl": ["1sten", "1ksien"],
        "gen-pl-poss": ["1ste", "1ksie"],
        "ptv-pl": "1ksi2",
        "ine-pl": "1ksiss2",
        "ela-pl": "1ksist2",
        "ill-pl": "1ksiin",
        "ill-pl-poss": "1ksii",
        "ade-pl": "1ksill2",
        "abl-pl": "1ksilt2",
        "all-pl": "1ksille",
        "ess-pl": "1ksin2",
        "tra-pl": "1ksiksi",
        "tra-pl-poss": "1ksikse",
        "ins-pl": "1ksin",
        "ins-pl-poss": None,
        "abe-pl": "1ksitt2",
        "cmt-pl": "1ksine",
        "comp": "1ksempi",
        "sup": "1ksin",
    },
    "fi-decl-kalleus": {
    	"nargs": 2,
        "nom-sg": "1s",
        "nom-sg-poss": "1te",
        "gen-sg": "1den",
        "gen-sg-poss": "1te",
        "ptv-sg": "1tt2",
        "ine-sg": "1dess2",
        "ela-sg": "1dest2",
        "ill-sg": "1teen",
        "ill-sg-poss": "1tee",
        "ade-sg": "1dell2",
        "abl-sg": "1delt2",
        "all-sg": "1delle",
        "ess-sg": "1ten2",
        "tra-sg": "1deksi",
        "tra-sg-poss": "1dekse",
        "ins-sg": None,
        "abe-sg": "1dett2",
        "cmt-sg": None,
        "nom-pl": "1det",
        "nom-pl-poss": "1te",
        "gen-pl": ["1ksien"],
        "gen-pl-poss": ["1ksie"],
        "ptv-pl": "1ksi2",
        "ine-pl": "1ksiss2",
        "ela-pl": "1ksist2",
        "ill-pl": "1ksiin",
        "ill-pl-poss": "1ksii",
        "ade-pl": "1ksill2",
        "abl-pl": "1ksilt2",
        "all-pl": "1ksille",
        "ess-pl": "1ksin2",
        "tra-pl": "1ksiksi",
        "tra-pl-poss": "1ksikse",
        "ins-pl": "1ksin",
        "ins-pl-poss": None,
        "abe-pl": "1ksitt2",
        "cmt-pl": "1ksine",
        "comp": "1ksempi",
        "sup": "1ksin",
    },
    "fi-decl-vieras": {
    	"nargs": 5,
        "nom-sg": "124s",
        "nom-sg-poss": "1344",
        "gen-sg": "1344n",
        "gen-sg-poss": "1344",
        "ptv-sg": "124st5",
        "ine-sg": "1344ss5",
        "ela-sg": "1344st5",
        "ill-sg": "1344seen",
        "ill-sg-poss": "1344see",
        "ade-sg": "1344ll5",
        "abl-sg": "1344lt5",
        "all-sg": "1344lle",
        "ess-sg": "1344n5",
        "tra-sg": "1344ksi",
        "tra-sg-poss": "1344kse",
        "ins-sg": None,
        "abe-sg": "1344tt5",
        "cmt-sg": None,
        "nom-pl": "1344t",
        "nom-pl-poss": "1344",
        "gen-pl": ["134iden", "134itten"],
        "gen-pl-poss": ["134ide", "134itte"],
        "ptv-pl": "134it5",
        "ine-pl": "134iss5",
        "ela-pl": "134ist5",
        "ill-pl": ["134isiin", "134ihin"],  # Latter rare
        "ill-pl-poss": ["134isii", "134ihi"],  # Latter rare
        "ade-pl": "134ill5",
        "abl-pl": "134ilt5",
        "all-pl": "134ille",
        "ess-pl": "134in5",
        "tra-pl": "134iksi",
        "tra-pl-poss": "134ikse",
        "ins-pl": "134in",
        "ins-pl-poss": None,
        "abe-pl": "134itt5",
        "cmt-pl": "134ine",
        "comp": "1344mpi",
        "sup": "134in",
    },
    "fi-decl-mies": {
    	"nargs": 0,
        "nom-sg": "1mies",
        "nom-sg-poss": "1miehe",
        "gen-sg": "1miehen",
        "gen-sg-poss": "1miehe",
        "ptv-sg": "1miestä",
        "ine-sg": "1miehessä",
        "ela-sg": "1miehestä",
        "ill-sg": "1mieheen",
        "ill-sg-poss": "1miehee",
        "ade-sg": "1miehellä",
        "abl-sg": "1mieheltä",
        "all-sg": "1miehelle",
        "ess-sg": "1miehenä",
        "tra-sg": "1mieheksi",
        "tra-sg-poss": "1miehekse",
        "ins-sg": None,
        "abe-sg": "1miehettä",
        "cmt-sg": None,
        "nom-pl": "1miehet",
        "nom-pl-poss": "1miehe",
        "gen-pl": ["1miesten", "1miehien"],
        "gen-pl-poss": ["1mieste", "1miehie"],
        "ptv-pl": "1miehiä",
        "ine-pl": "1miehissä",
        "ela-pl": "1miehistä",
        "ill-pl": "1miehiin",
        "ill-pl-poss": "1miehii",
        "ade-pl": "1miehillä",
        "abl-pl": "1miehiltä",
        "all-pl": "1miehille",
        "ess-pl": "1miehinä",
        "tra-pl": "1miehiksi",
        "tra-pl-poss": "1miehikse",
        "ins-pl": "1miehin",
        "ins-pl-poss": None,
        "abe-pl": "1miehittä",
        "cmt-pl": "1miehine",
        "comp": "1miehempi",
        "sup": "1miehin",
    },
    "fi-decl-ohut": {
    	"nargs": 5,
        "nom-sg": "124t",
        "nom-sg-poss": "134e",
        "gen-sg": "134en",
        "gen-sg-poss": "134e",
        "ptv-sg": "124tt5",
        "ine-sg": "134ess5",
        "ela-sg": "134est5",
        "ill-sg": "134een",
        "ill-sg-poss": "134ee",
        "ade-sg": "134ell5",
        "abl-sg": "134elt5",
        "all-sg": "134elle",
        "ess-sg": "134en5",
        "tra-sg": "134eksi",
        "tra-sg-poss": "134ekse",
        "ins-sg": None,
        "abe-sg": "134ett5",
        "cmt-sg": None,
        "nom-pl": "134et",
        "nom-pl-poss": "134e",
        "gen-pl": ["134iden", "134itten"],
        "gen-pl-poss": ["134ide", "134itte"],
        "ptv-pl": "134it5",
        "ine-pl": "134iss5",
        "ela-pl": "134ist5",
        "ill-pl": ["134isiin", "134ihin"],
        "ill-pl-poss": ["134isii", "134ihi"],
        "ade-pl": "134ill5",
        "abl-pl": "134ilt5",
        "all-pl": "134ille",
        "ess-pl": "134in5",
        "tra-pl": "134iksi",
        "tra-pl-poss": "134ikse",
        "ins-pl": "134in",
        "ins-pl-poss": None,
        "abe-pl": "134itt5",
        "cmt-pl": "134ine",
        "comp": "134empi",
        "sup": "134in",
    },
    "fi-decl-kevät": {
    	"nargs": 2,
        "nom-sg": "1t",
        "nom-sg-poss": "12",
        "gen-sg": "12n",
        "gen-sg-poss": "12",
        "ptv-sg": ["1tt2"],
        "ine-sg": "12ss2",
        "ela-sg": "12st2",
        "ill-sg": "12seen",
        "ill-sg-poss": "12see",
        "ade-sg": "12ll2",
        "abl-sg": "12lt2",
        "all-sg": "12lle",
        "ess-sg": "12n2",
        "tra-sg": "12ksi",
        "tra-sg-poss": "12kse",
        "ins-sg": None,
        "abe-sg": "12tt2",
        "cmt-sg": None,
        "nom-pl": "1t",
        "nom-pl-poss": "12",
        "gen-pl": ["1iden", "1itten"],
        "gen-pl-poss": ["1ide", "1itte"],
        "ptv-pl": ["1it2"],
        "ine-pl": ["1iss2"],
        "ela-pl": ["1ist2"],
        "ill-pl": ["1isiin", "1ihin"],  # Last rare
        "ill-pl-poss": ["1isii", "1ihi"],  # Last rare
        "ade-pl": ["1ill2"],
        "abl-pl": ["1ilt2"],
        "all-pl": ["1ille"],
        "ess-pl": ["1in2"],
        "tra-pl": ["1iksi"],
        "tra-pl-poss": ["1ikse"],
        "ins-pl": ["1in"],
        "ins-pl-poss": None,
        "abe-pl": ["1itt2"],
        "cmt-pl": ["1ine"],
        "comp": "12mpi",
        "sup": "1in",
    },
    "fi-decl-kahdeksas": {
    	"nargs": 2,
        "nom-sg": "1s",
        "nom-sg-poss": "1nte",
        "gen-sg": "1nnen",
        "gen-sg-poss": "1nte",
        "ptv-sg": "1tt2",
        "ine-sg": "1nness2",
        "ela-sg": "1nnest2",
        "ill-sg": "1nteen",
        "ill-sg-poss": "1ntee",
        "ade-sg": "1nnell2",
        "abl-sg": "1nnelt2",
        "all-sg": "1nnelle",
        "ess-sg": "1nten2",
        "tra-sg": "1nneksi",
        "tra-sg-poss": "1nnekse",
        "ins-sg": None,
        "abe-sg": "1nnett2",
        "cmt-sg": None,
        "nom-pl": "1nnet",
        "nom-pl-poss": "1nte",
        "gen-pl": ["1nsien"],
        "gen-pl-poss": ["1nsie"],
        "ptv-pl": "1nsi2",
        "ine-pl": "1nsiss2",
        "ela-pl": "1nsist2",
        "ill-pl": "1nsiin",
        "ill-pl-poss": "1nsii",
        "ade-pl": "1nsill2",
        "abl-pl": "1nsilt2",
        "all-pl": "1nsille",
        "ess-pl": "1nsin2",
        "tra-pl": "1nsiksi",
        "tra-pl-poss": "1nsikse",
        "ins-pl": "1nsin",
        "ins-pl-poss": None,
        "abe-pl": "1nsitt2",
        "cmt-pl": "1nsine",
        "comp": "1mpi",
        "sup": "1-in",
    },
    "fi-decl-tuhat": {
    	"nargs": 2,
        "nom-sg": "1t",
        "nom-sg-poss": "1nte",
        "gen-sg": "1nnen",
        "gen-sg-poss": "1nte",
        "ptv-sg": "1tt2",
        "ine-sg": "1nness2",
        "ela-sg": "1nnest2",
        "ill-sg": "1nteen",
        "ill-sg-poss": "1ntee",
        "ade-sg": "1nnell2",
        "abl-sg": "1nnelt2",
        "all-sg": "1nnelle",
        "ess-sg": "1nten2",
        "tra-sg": "1nneksi",
        "tra-sg-poss": "1nnekse",
        "ins-sg": None,
        "abe-sg": "1nnett2",
        "cmt-sg": None,
        "nom-pl": "1nnet",
        "nom-pl-poss": "1nte",
        "gen-pl": ["1nsien", "1nten"],  # Latter rare
        "gen-pl-poss": ["1nsie", "1nte"],  # Latter rare
        "ptv-pl": "1nsi2",
        "ine-pl": "1nsiss2",
        "ela-pl": "1nsist2",
        "ill-pl": "1nsiin",
        "ill-pl-poss": "1nsii",
        "ade-pl": "1nsill2",
        "abl-pl": "1nsilt2",
        "all-pl": "1nsille",
        "ess-pl": "1nsin2",
        "tra-pl": "1nsiksi",
        "tra-pl-poss": "1nsikse",
        "ins-pl": "1nsin",
        "ins-pl-poss": None,
        "abe-pl": "1nsitt2",
        "cmt-pl": "1nsine",
        "comp": "1nnempi",
        "sup": "1nsin",
    },
    "fi-decl-kuollut": {
    	"nargs": 2,
        "nom-sg": "12-Ut",
        "nom-sg-poss": "1ee",
        "gen-sg": "1een",
        "gen-sg-poss": "1ee",
        "ptv-sg": "12-Utt2",
        "ine-sg": "1eess2",
        "ela-sg": "1eest2",
        "ill-sg": "1eeseen",
        "ill-sg-poss": "1eesee",
        "ade-sg": "1eell2",
        "abl-sg": "1eelt2",
        "all-sg": "1eelle",
        "ess-sg": "1een2",
        "tra-sg": "1eeksi",
        "tra-sg-poss": "1eekse",
        "ins-sg": None,
        "abe-sg": "1eett2",
        "cmt-sg": None,
        "nom-pl": "1eet",
        "nom-pl-poss": "1ee",
        "gen-pl": ["1eiden", "1eitten"],  # Latter rare
        "gen-pl-poss": ["1eide", "1eitte"],  # Latter rare
        "ptv-pl": "1eit2",
        "ine-pl": "1eiss2",
        "ela-pl": "1eist2",
        "ill-pl": ["1eisiin", "1eihin"],
        "ill-pl-poss": ["1eisii", "1eihi"],
        "ade-pl": "1eill2",
        "abl-pl": "1eilt2",
        "all-pl": "1eille",
        "ess-pl": "1ein2",
        "tra-pl": "1eiksi",
        "tra-pl-poss": "1eikse",
        "ins-pl": "1ein",
        "ins-pl-poss": None,
        "abe-pl": "1eitt2",
        "cmt-pl": "1eine",
        "comp": "1eempi",
        "sup": "1ein",
    },
    "fi-decl-hame": {
    	"nargs": 4,
        "nom-sg": "12e",
        "nom-sg-poss": "13ee",
        "gen-sg": "13een",
        "gen-sg-poss": "13ee",
        "ptv-sg": "12ett4",
        "ine-sg": "13eess4",
        "ela-sg": "13eest4",
        "ill-sg": "13eeseen",
        "ill-sg-poss": "13eesee",
        "ade-sg": "13eell4",
        "abl-sg": "13eelt4",
        "all-sg": "13eelle",
        "ess-sg": "13een4",
        "tra-sg": "13eeksi",
        "tra-sg-poss": "13eekse",
        "ins-sg": None,
        "abe-sg": "13eett4",
        "cmt-sg": None,
        "nom-pl": "13eet",
        "nom-pl-poss": "13ee",
        "gen-pl": ["13eiden", "13eitten"],
        "gen-pl-poss": ["13eide", "13eitte"],
        "ptv-pl": "13eit4",
        "ine-pl": "13eiss4",
        "ela-pl": "13eist4",
        "ill-pl": ["13eisiin", "13eihin"],
        "ill-pl-poss": ["13eisii", "13eihi"],
        "ade-pl": "13eill4",
        "abl-pl": "13eilt4",
        "all-pl": "13eille",
        "ess-pl": "13ein4",
        "tra-pl": "13eiksi",
        "tra-pl-poss": "13eikse",
        "ins-pl": "13ein",
        "ins-pl-poss": None,
        "abe-pl": "13eitt4",
        "cmt-pl": "13eine",
        "comp": "13eempi",
        "sup": "13ein",
    },

    "fi-decl-askel": {
    	"nargs": 5,
        "nom-sg": ["124", "134e"],
        "nom-sg-poss": ["134e", "134ee"],
        "gen-sg": ["134en", "134een"],
        "gen-sg-poss": ["134e", "134ee"],
        "ptv-sg": ["124t5", "134ett5"],
        "ine-sg": ["134ess5", "134eess5"],
        "ela-sg": ["134est5", "134eest5"],
        "ill-sg": ["134een", "134eeseen"],
        "ill-sg-poss": ["134ee", "134eesee"],
        "ade-sg": ["134ell5", "134eell5"],
        "abl-sg": ["134elt5", "134eelt5"],
        "all-sg": ["134elle", "134eelle"],
        "ess-sg": ["134en5", "134een5"],
        "tra-sg": ["134eksi", "134eeksi"],
        "tra-sg-poss": ["134ekse", "134eekse"],
        "ins-sg": None,
        "abe-sg": ["134ett5", "134eett5"],
        "cmt-sg": None,
        "nom-pl": ["134et", "134eet"],
        "nom-pl-poss": ["134e", "134ee"],
        "gen-pl": ["134ien", "124ten", "134eiden", "134eitten"],
        "gen-pl-poss": ["134ie", "124te", "134eide", "134eitte"],
        "ptv-pl": ["134i5", "134eit5"],
        "ine-pl": ["134iss5", "134eiss5"],
        "ela-pl": ["134ist5", "134eist5"],
        "ill-pl": ["134iin", "134eisiin"],
        "ill-pl-poss": ["134ii", "134eisii"],
        "ade-pl": ["134ill5", "134eill5"],
        "abl-pl": ["134ilt5", "134eilt5"],
        "all-pl": ["134ille", "134eille"],
        "ess-pl": ["134in5", "134ein5"],
        "tra-pl": ["134iksi", "134eiksi"],
        "tra-pl-poss": ["134ikse", "134eikse"],
        "ins-pl": ["134in", "134ein"],
        "ins-pl-poss": None,
        "abe-pl": ["134itt5", "134eitt5"],
        "cmt-pl": ["134ine", "134eine"],
        "comp": ["134empi", "134eempi"],  # Not used?
        "sup": ["134in", "134ein"],  # Not used?
    },

    "fi-decl-filee": {
    	"nargs": 2,
        "nom-sg": "1",
        "gen-sg": "1n",
        "gen-sg-poss": "1",
        "ptv-sg": ["1t2"],
        "ine-sg": "1ss2",
        "ela-sg": "1st2",
        "ill-sg": ["1h@n", "seen"],
        "ill-sg-poss": ["1h@", "see"],
        "ade-sg": "1ll2",
        "abl-sg": "1lt2",
        "all-sg": "1lle",
        "ess-sg": "1n2",
        "tra-sg": "1ksi",
        "tra-sg-poss": "1kse",
        "ins-sg": None,
        "abe-sg": "1tt2",
        "cmt-sg": None,
        "nom-pl": "1t",
        "nom-pl-poss": "1",
        "gen-pl": ["1-iden", "1-itten"],
        "gen-pl-poss": ["1-ide", "1-itte"],
        "ptv-pl": ["1-it2"],
        "ine-pl": ["1-iss2"],
        "ela-pl": ["1-ist2"],
        "ill-pl": ["1-ihin", "1-isiin"],
        "ill-pl-poss": ["1-ihi", "1-isii"],
        "ade-pl": ["1-ill2"],
        "abl-pl": ["1-ilt2"],
        "all-pl": ["1-ille"],
        "ess-pl": ["1-in2"],
        "tra-pl": ["1-iksi"],
        "tra-pl-poss": ["1-ikse"],
        "ins-pl": ["1-in"],
        "ins-pl-poss": None,
        "abe-pl": ["1-itt2"],
        "cmt-pl": ["1-ine"],
        "comp": "1mpi",  # Not used?
        "sup": "1-in",  # Not used?
    },

    "fi-decl-maa-dot": {
    	"nargs": 3,
        "nom-sg": "1",
	"nom-sg-clitic": "1:",
	"nom-sg-poss": "1:",
        "gen-sg": "1:n",
        "gen-sg-poss": "1:",
        "ptv-sg": ["1:t3", "1:33"],  #eg. flmnrsx(z?) endings often take latter?
        "ine-sg": "1:ss3",
        "ela-sg": "1:st3",
        "ill-sg": "1:h2n",
        "ill-sg-poss": "1:h2",
        "ade-sg": "1:ll3",
        "abl-sg": "1:lt3",
        "all-sg": "1:lle",
        "ess-sg": "1:n3",
        "tra-sg": "1:ksi",
        "tra-sg-poss": "1:kse",
        "ins-sg": None,
        "abe-sg": "1:tt3",
        "cmt-sg": None,
        "nom-pl": "1:t",
        "nom-pl-poss": "1:",
        "gen-pl": ["1:iden", "1:itten"],
        "gen-pl-poss": ["1:ide", "1:itte"],
        "ptv-pl": ["1:it3"],
        "ine-pl": ["1:iss3"],
        "ela-pl": ["1:ist3"],
        "ill-pl": ["1:ihin"],
        "ill-pl-poss": ["1:ihi"],
        "ade-pl": ["1:ill3"],
        "abl-pl": ["1:ilt3"],
        "all-pl": ["1:ille"],
        "ess-pl": ["1:in3"],
        "tra-pl": ["1:iksi"],
        "tra-pl-poss": ["1:ikse"],
        "ins-pl": ["1:in"],
        "ins-pl-poss": None,
        "abe-pl": ["1:itt3"],
        "cmt-pl": ["1:ine"],
        "comp": "1:mpi",  # Not used?
        "sup": "1:in",  # Not used?
    },

    "fi-decl-filee-dot": {
    	"nargs": 3,
        "nom-sg": "1",
	"nom-sg-clitic": "1:",
	"nom-sg-poss": "1:",
        "gen-sg": "1:n",
        "gen-sg-poss": "1",
        "ptv-sg": ["1:t3"],
        "ine-sg": "1:ss3",
        "ela-sg": "1:st3",
        "ill-sg": ["1:h2n", "1:seen"],
        "ill-sg-poss": ["1:h2", "1:see"],
        "ade-sg": "1:ll3",
        "abl-sg": "1:lt3",
        "all-sg": "1:lle",
        "ess-sg": "1:n3",
        "tra-sg": "1:ksi",
        "tra-sg-poss": "1:kse",
        "ins-sg": None,
        "abe-sg": "1:tt3",
        "cmt-sg": None,
        "nom-pl": "1:t",
        "nom-pl-poss": "1",
        "gen-pl": ["1:iden", "1:itten"],
        "gen-pl-poss": ["1:ide", "1:itte"],
        "ptv-pl": ["1:it3"],
        "ine-pl": ["1:iss3"],
        "ela-pl": ["1:ist3"],
        "ill-pl": ["1:ihin", "1:isiin"],
        "ill-pl-poss": ["1:ihi", "1:isii"],
        "ade-pl": ["1:ill3"],
        "abl-pl": ["1:ilt3"],
        "all-pl": ["1:ille"],
        "ess-pl": ["1:in3"],
        "tra-pl": ["1:iksi"],
        "tra-pl-poss": ["1:ikse"],
        "ins-pl": ["1:in"],
        "ins-pl-poss": None,
        "abe-pl": ["1:itt3"],
        "cmt-pl": ["1:ine"],
        "comp": "1:mpi",  # Not used?
        "sup": "1:in",  # Not used?
    },

    "fi-decl-käsi-kulkija": {
        "split": (1, "fi-decl-käsi", 3, "fi-decl-kulkija"),
    },

    "fi-decl-käsi-maa": {
        "split": (1, "fi-decl-käsi", 3, "fi-decl-maa"),
    },

    "fi-decl-käsi-käsi": {
        "split": (1, "fi-decl-käsi", 4, "fi-decl-käsi"),
    },

    "fi-decl-käsi-risti": {
        "split": (1, "fi-decl-käsi", 3, "fi-decl-risti"),
    },

    "fi-decl-onneton-risti": {
        "split": (1, "fi-decl-onneton", 4, "fi-decl-risti"),
    },

    "fi-decl-koira-kala": {
        "split": (1, "fi-decl-koira", 5, "fi-decl-kala"),
    },

    "fi-decl-pieni-uni": {
        "split": (1, "fi-decl-pieni", 3, "fi-decl-uni"),
    },

    "fi-decl-valo-koira": {
        "split": (1, "fi-decl-valo", 6, "fi-decl-koira"),
    },

    "fi-decl-koira-valo": {
        "split": (1, "fi-decl-koira", 5, "fi-decl-valo"),
    },

    "fi-decl-koira-ovi": {
        "split": (1, "fi-decl-koira", 5, "fi-decl-ovi"),
    },

    "fi-decl-koira-koira": {
        "split": (1, "fi-decl-koira", 5, "fi-decl-koira"),
    },

    "fi-decl-koira-uni": {
        "split": (1, "fi-decl-koira", 6, "fi-decl-uni"),
    },

    "fi-decl-nainen-koira": {
        "split": (1, "fi-decl-nainen", 3, "fi-decl-koira"),
    },

    "fi-decl-lämmin-koira": {
        "split": (1, "fi-decl-lämmin", 2, "fi-decl-koira"),
    },

    "fi-decl-lämmin-palvelu": {
        "split": (1, "fi-decl-lämmin", 2, "fi-decl-palvelu"),
    },

    "fi-decl-lämmin-valo": {
        "split": (1, "fi-decl-lämmin", 2, "fi-decl-valo"),
    },

    "fi-decl-valo-valo": {
        "split": (1, "fi-decl-valo", 6, "fi-decl-valo"),
    },

    "fi-decl-valo-kala": {
        "split": (1, "fi-decl-valo", 6, "fi-decl-kala"),
    },

    "fi-decl-valo-ovi": {
        "split": (1, "fi-decl-valo", 6, "fi-decl-ovi"),
    },

    "fi-decl-kala-ovi": {
        "split": (1, "fi-decl-kala", 6, "fi-decl-ovi"),
    },

    "fi-decl-kala-risti": {
        "split": (1, "fi-decl-kala", 6, "fi-decl-risti"),
    },

    "fi-decl": {
        # This declension is used for irregular nominals, and all forms are
	# expressly stated in the template.
    },

    "fi-decl-pron": {
        # This declension is used for pronouns (irregular), and all forms are
	# expressly stated in the template.  This template uses
	# unusual names for the forms, and a mapping is used to map names we
	# use normally into those.
    },
}

possessive_suffixes = {
    "1s": "ni",
    "2s": "si",
    "3x": ["@n", "nsA"],
    "1p": "mme",
    "2p": "nne",
}



# This file defines the public exports from the wiktfinnish module.
#
# Copyright (c) 2018 Tatu Ylonen.  See LICENSE.

from wiktfinnish.formnames import COMP_FORMS, CASE_FORMS, POSSESSIVE_FORMS
from wiktfinnish.formnames import VERB_FORMS, CLITIC_FORMS
from wiktfinnish.formnames import all_forms_list, all_forms_iter
from wiktfinnish.inflect import inflect
from wiktfinnish.inflect import add_clitic

__all__ = (
    "inflect",
    "add_clitic",
    "COMP_FORMS",
    "CASE_FORMS",
    "POSSESSIVE_FORMS",
    "VERB_FORMS",
    "CLITIC_FORMS",
    "all_forms_list",
    "all_forms_iter",
)
